package logic;

import exceptions.SongNotInListException;
import javafx.beans.property.*;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.scene.image.Image;

import java.util.LinkedList;

public class SongCollection {
    private LinkedList<Song> songList;

    private IntegerProperty currentSongProperty;
    private StringProperty currentArtistProperty;
    private StringProperty currentSongNameProperty;
    private ObjectProperty<Image> currentCoverProperty;
    private StringProperty currentSongURLProperty;


    public SongCollection(final LinkedList<Song> songList) {
        this.songList = songList;

        initProperties();
        setCurrentSongToFirstSong();
    }

    private void initProperties() {
        // -1 because setting later to 0 should result in a change, i.e. firing a ChangeEvent
        this.currentSongProperty = new SimpleIntegerProperty(-1);
        this.currentArtistProperty = new SimpleStringProperty();
        this.currentSongNameProperty = new SimpleStringProperty();
        this.currentCoverProperty = new SimpleObjectProperty<Image>();
        this.currentSongURLProperty = new SimpleStringProperty();

        this.currentSongProperty.addListener(new ChangeListener() {
            @Override
            public void changed(ObservableValue observable, Object oldValue, Object newValue) {
                currentArtistProperty.setValue(songList.get(currentSongProperty.getValue()).getArtist());
                currentSongNameProperty.setValue(songList.get(currentSongProperty.getValue()).getSongName());
                currentCoverProperty.setValue(songList.get(currentSongProperty.getValue()).getCover());
                currentSongURLProperty.setValue(songList.get(currentSongProperty.getValue()).getSongURL());
            }
        });
    }

    public void setCurrentSongToFirstSong() {
        currentSongProperty.setValue(0);
    }

    public void addSong(Song song) {
        this.songList.add(song);
    }

    public void removeSong(Song song) throws SongNotInListException {
        if (songList.contains(song)) {
            int index = songList.indexOf(song);
            songList.remove(index);

            int currentSongIndex = currentSongProperty.getValue();
            if (index <= currentSongIndex) {
                currentSongProperty.setValue(currentSongIndex - 1);
            }
        } else {
            throw new SongNotInListException(song.getSongName(), song.getArtist());
        }
    }

    public void nextSong() {
        if (songList.size() > 0) {
            int nextSong = currentSongProperty.getValue() + 1;
            int songCount = songList.size();

            if (nextSong == songCount) {
                // if forwarding last song, skip to first song
                currentSongProperty.setValue(0);
            } else if (nextSong < songCount) {
                currentSongProperty.setValue(nextSong);
            }
        }
    }

    public void previousSong() {
        if (songList.size() > 0) {
            int nextSong = currentSongProperty.getValue() - 1;
            int songCount = songList.size();

            if (nextSong == -1) {
                // if backwarding first song, skip to last song
                currentSongProperty.setValue(songCount - 1);
            } else if ((nextSong >= 0) && (nextSong < songCount)) {
                currentSongProperty.setValue(nextSong);
            }
        }
    }

    public StringProperty getCurrentSongNameProperty() {
        return currentSongNameProperty;
    }

    public ObjectProperty<Image> getCurrentCoverProperty() {
        return currentCoverProperty;
    }

    public StringProperty getCurrentArtistProperty() {
        return currentArtistProperty;
    }

    public StringProperty getCurrentSongURLProperty() {
        return currentSongURLProperty;
    }
}
